<?php
#############################################################################
#                SuperMailingList / SuperWebMailer                          #
#               Copyright  2007 - 2024 Mirko Boeer                         #
#                    Alle Rechte vorbehalten.                               #
#                http://www.supermailinglist.de/                            #
#                http://www.superwebmailer.de/                              #
#   Support SuperMailingList: support@supermailinglist.de                   #
#   Support SuperWebMailer: support@superwebmailer.de                       #
#   Support-Forum: http://board.superscripte.de/                            #
#                                                                           #
#   Dieses Script ist urheberrechtlich geschuetzt. Zur Nutzung des Scripts  #
#   muss eine Lizenz erworben werden.                                       #
#                                                                           #
#   Das Script darf weder als ganzes oder als Teil eines anderen Projekts   #
#   verwendet oder weiterverkauft werden.                                   #
#                                                                           #
#   Beachten Sie fuer den Einsatz des Script-Pakets die Lizenzbedingungen   #
#                                                                           #
#   Fuehren Sie keine Veraenderungen an diesem Script durch. Jegliche       #
#   Veraenderungen koennen nicht supported werden.                          #
#                                                                           #
#############################################################################

  if( !(version_compare(PHP_VERSION, "8.0.0") >= 0) ){
    print "requires PHP 8.0 and newer, your version " . PHP_VERSION;
    die;
  } 

  // https://github.com/chillerlan/php-qrcode/tree/main
  //https://php-qrcode.readthedocs.io/en/main/
  use chillerlan\QRCode\{QRCode, QROptions};
  use chillerlan\QRCode\Data\QRMatrix;
  use chillerlan\QRCode\Output\QROutputInterface;
  use chillerlan\QRCode\Common\Version;
  use chillerlan\QRCode\Common\EccLevel;

  require_once __DIR__ . '/vendor/autoload.php';

  function CreateQRCode($output_text, $width = 100, $height = 100, $outputType = "png", $outputBase64 = false, $outputBase64AsIMGURL = true, $sendContentTypeHeader = true, $eccLevel = "Q", $quietzoneSize = 1){

    if($width < 0)
       $width = 100;
       
    if($height <> $width)
       $height = $width;
    
    $options = new QROptions;

    $options->version             = Version::AUTO;
    switch ($eccLevel){
      case "L":
        $options->eccLevel            = EccLevel::L;
        break;
      case "M":
        $options->eccLevel            = EccLevel::M;
        break;
      case "H":
        $options->eccLevel            = EccLevel::H;
        break;
      default:
        $options->eccLevel            = EccLevel::Q; //L, M, Q, H
    }

    $outputType = strtolower($outputType);
    $options->outputType = QROutputInterface::GDIMAGE_PNG;
    if($outputType == "png")
      $options->outputType          = QROutputInterface::GDIMAGE_PNG;
      else
      if($outputType == "jpeg" || $outputType == "jpg")
       $options->outputType          = QROutputInterface::GDIMAGE_JPG;
       else
       if($outputType == "gif")
        $options->outputType          = QROutputInterface::GDIMAGE_GIF;

    $options->scale               = 20;

    $options->quality             = 50;

    $options->outputBase64        = false;

    $options->quietzoneSize = $quietzoneSize;

    $options->returnResource = true; //=> GdImage instance

    $out_image = (new QRCode($options))->render($output_text);

    $new_image = imagescale(
        $out_image,
        $width,
        $height,
        IMG_BILINEAR_FIXED
    );

    
   if($sendContentTypeHeader){
      // Prevent the browser from caching the result.
      // Date in the past
      header('Expires: Mon, 26 Jul 1997 05:00:00 GMT') ;
      // always modified
      header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT') ;
      // HTTP/1.1
      header('Cache-Control: no-store, no-cache, must-revalidate') ;
      header('Cache-Control: post-check=0, pre-check=0', false) ;
      // HTTP/1.0
      header('Pragma: no-cache') ;

      if(!$outputBase64)
        header('Content-type: image/' . $outputType);
        else
        header('Content-type: text/plain');
   }   

    ob_start();

    if($options->outputType == QROutputInterface::GDIMAGE_PNG)
      imagepng($new_image);
      else
      if($options->outputType == QROutputInterface::GDIMAGE_JPG)
        imagejpeg($new_image);
        else
        if($options->outputType == QROutputInterface::GDIMAGE_GIF)
          imagegif($new_image);

    $image_data = ob_get_contents();
    ob_end_clean();


    imagedestroy($new_image);
    imagedestroy($out_image);

    if($outputBase64){
     if(!$outputBase64AsIMGURL)
       return base64_encode($image_data);
       else{
         return 'data:image/' . $outputType . ';base64,' . base64_encode($image_data);
       }

    }

    return $image_data;
  }


  /*header('Content-type: image/png');

  $x = CreateQRCode("http://www.superwebmailer.de", 200, 200, "png", false);
  print $x;

  die;

  
  header('Content-type: text/html');


  $x = CreateQRCode("http://www.superwebmailer.de", 200, 200, "png", true);

  print '<p><img src="' . $x . '" alt="image 1" width="200" height="200"/></p>';
  */


?>