<?php
#############################################################################
#                SuperMailingList / SuperWebMailer                          #
#               Copyright  2007 - 2024 Mirko Boeer                         #
#                    Alle Rechte vorbehalten.                               #
#                http://www.supermailinglist.de/                            #
#                http://www.superwebmailer.de/                              #
#   Support SuperMailingList: support@supermailinglist.de                   #
#   Support SuperWebMailer: support@superwebmailer.de                       #
#   Support-Forum: http://board.superscripte.de/                            #
#                                                                           #
#   Dieses Script ist urheberrechtlich geschuetzt. Zur Nutzung des Scripts  #
#   muss eine Lizenz erworben werden.                                       #
#                                                                           #
#   Das Script darf weder als ganzes oder als Teil eines anderen Projekts   #
#   verwendet oder weiterverkauft werden.                                   #
#                                                                           #
#   Beachten Sie fuer den Einsatz des Script-Pakets die Lizenzbedingungen   #
#                                                                           #
#   Fuehren Sie keine Veraenderungen an diesem Script durch. Jegliche       #
#   Veraenderungen koennen nicht supported werden.                          #
#                                                                           #
#############################################################################


use SPFLib\Checker;
use SPFLib\Check\Environment;
use SPFLib\DNS;

/* IdnaMap.php 

added

protected static $validSingle = array(

95 => 1, // M.B. SPF records _ is valid 

Checker.php

            case Result::CODE_ERROR_PERMANENT:
                return false;// M.B. no Exception here => throw new Exception\IncludeMechanismException(Result::CODE_ERROR_PERMANENT, $domain, $mechanism, $includeResult);

*/

require('vendor/autoload.php');

function DMARCCheck($senderemailaddress){
  if(strpos($senderemailaddress, "@") !== false)
    $senderdomain = substr($senderemailaddress, strpos($senderemailaddress, "@") + 1);
    else
    $senderdomain = $senderemailaddress;

  $resolver = new SPFLib\DNS\StandardResolver;
  try {
  $txtRecords = $resolver->getTXTRecords("_dmarc." . $senderdomain); // check for _dmarc.superwebmailer.de
  } catch (\SPFLib\Exception $x) {
      // Problems retrieving text record from example.com,
      // or problems decoding it
      $txtRecords = array();
  }
  return count($txtRecords) > 0;
}

function SPFRecordCheck($smtp_servername, $smtp_heloname, $senderemailaddress){
 
  $CRLF = "<br />\r\n";
  if(empty($smtp_heloname) && isset($smtp_servername)) $smtp_heloname = $smtp_servername; 
 

  $results = $CRLF . $CRLF;
  
  $results .= "SPF check results ------------" . $CRLF. $CRLF;
  
  if(empty($senderemailaddress)){
    $results .= "no sender/return-path email address" . $CRLF . $CRLF;
    return $results;
  }

  $senderdomain = substr($senderemailaddress, strpos($senderemailaddress, "@") + 1);

  if(empty($smtp_servername)){
    $results .= "no smtp server" . $CRLF . $CRLF;
    
    _DMARCcheck($results, $senderemailaddress, $senderdomain);
    
    return $results;
  }

    // get spf record

    $decoder = new \SPFLib\Decoder();
    $record = null;
    try {
        $record = $decoder->getRecordFromDomain($senderdomain);
    } catch (\SPFLib\Exception $x) {
        // Problems retrieving the SPF record from example.com,
        // or problems decoding it
        $record = null;
    }

    if ($record === null) {
        // SPF record not found for example.com
        $results .= "no spf record for domain $senderdomain, mail from email address &lt;$senderemailaddress&gt;" . $CRLF;
        return $results;
    }

    $results .=  "smtp mail from: " . $senderemailaddress . $CRLF;
    $results .=  $CRLF;

    if ($record !== null) {

      $results .=  "SPF record for $senderdomain: ";

      // List all terms (that is, mechanisms and modifiers)
      foreach ($record->getTerms() as $term) {
          $results .=  $term . " ";
      }

      $results .=  $CRLF . $CRLF;
    }


    // resolve ip(s)

    $resolver = new SPFLib\DNS\StandardResolver;

    try {
      $ip = $resolver->getIPAddressesFromDomainName($smtp_servername);
    } catch(\SPFLib\Exception $x) {
      $ip = null;
    }

    $smtp_serverIPv4 = "";
    $smtp_serverIPv6 = "";

    if( isset($ip) ){

      for($i=0; $i<count($ip); $i++){
        if( $ip[$i]->getAddressType() == 4)
          $smtp_serverIPv4 = $ip[$i]->toString();
          else
          $smtp_serverIPv6 = $ip[$i]->toString();
      }
    }else{
      $results .= "no ip address for '$smtp_servername'";
      return $results;
    } 

    $results .=  "smtp server: " . $smtp_servername . $CRLF;
    $results .=  "&nbsp;&nbsp;smtp server IPv4: " . (!empty($smtp_serverIPv4) ? $smtp_serverIPv4 : 'none') . $CRLF;
    $results .=  "&nbsp;&nbsp;smtp server IPv6: " . (!empty($smtp_serverIPv6) ? $smtp_serverIPv6 : 'none') . $CRLF;

    $results .=  $CRLF . "smtp heloname: " . $smtp_heloname . $CRLF;

    $results .=  $CRLF . $CRLF;

    $results .= "Informational$CRLF<strong>";
    
    // check spf record
    // checking not really possible when smtp server ip <> outgoing server ip

    $checker = new \SPFLib\Checker();

    try {
                                                  //`1.2.3.4`, `helo.domain`, `sender@domain.com`
      $environment = new \SPFLib\Check\Environment($smtp_serverIPv4, $smtp_heloname, $senderemailaddress);
    } catch (\SPFLib\Exception $x) {
      return $results;
    }

    $checkResult = $checker->check($environment);
    $results .=  "Result IPv4 $smtp_serverIPv4: " . strtoupper( $checkResult->getCode() ) . " " . ( ($checkResult->getMatchedMechanism() !== null) ? $checkResult->getMatchedMechanism()->getQualifier(true) : "" ) . $CRLF;

    if(!empty($smtp_serverIPv6)){
       try {
         $environment = new \SPFLib\Check\Environment($smtp_serverIPv6, $smtp_heloname, $senderemailaddress);
       } catch (\SPFLib\Exception $x) {
         return $results;
       }
       $checkResult = $checker->check($environment);
       $results .=  "Result IPv6 $smtp_serverIPv6: " . strtoupper( $checkResult->getCode() ) . " " .  ( ($checkResult->getMatchedMechanism() !== null) ? $checkResult->getMatchedMechanism()->getQualifier(true) : "" ) . $CRLF;
    }

    $results .= "</strong>";
    
    $results .=  $CRLF . 'Interprete SPF record results: <a href="https://de.wikipedia.org/wiki/Sender_Policy_Framework" target="_blank">https://de.wikipedia.org/wiki/Sender_Policy_Framework</a> and <a href="https://en.wikipedia.org/wiki/Sender_Policy_Framework" target="_blank">https://en.wikipedia.org/wiki/Sender_Policy_Framework</a>.' . $CRLF . $CRLF;

    _DMARCcheck($results, $senderemailaddress, $senderdomain);
    
    return $results;
  
}

function _DMARCcheck(&$results, $senderemailaddress, $senderdomain){
    $CRLF = "<br />\r\n";
    $results .= "DMARC check results ------------" . $CRLF. $CRLF;
    if(!DMARCCheck($senderemailaddress)){
      $results .= "NO DMARC record for domain $senderdomain, mail from email address &lt;$senderemailaddress&gt;" . $CRLF;
    }else
      $results .= "DMARC record for domain $senderdomain, mail from email address &lt;$senderemailaddress&gt;, found." . $CRLF;
    $results .= $CRLF.$CRLF;  
 
}

function SPFRecordCheck_OnlySPFCheck($senderemailaddress){
 
  $CRLF = " ";

  $results = "";
 

  if(empty($senderemailaddress)){
    return $results;
  }

    // get spf record

    $senderdomain = substr($senderemailaddress, strpos($senderemailaddress, "@") + 1);
    $decoder = new \SPFLib\Decoder();
    $record = null;
    try {
        $record = $decoder->getRecordFromDomain($senderdomain);
    } catch (\SPFLib\Exception $x) {
        // Problems retrieving the SPF record from example.com,
        // or problems decoding it
        $record = null;
    }

    if ($record === null) {
        // SPF record not found for example.com
        $results .= "no spf record for domain $senderdomain, mail from email address &lt;$senderemailaddress&gt;" . $CRLF;
        
        if(!DMARCcheck($senderemailaddress)){
          $results .= "NO _DMARC record for domain $senderdomain, mail from email address &lt;$senderemailaddress&gt;" . $CRLF;
        }
        
        return $results;
    }

    if(!DMARCcheck($senderemailaddress)){

       if ($record !== null)
         $results .= "SPF record found for domain $senderdomain, mail from email address &lt;$senderemailaddress&gt;" . $CRLF. $CRLF . "<br /><br />";

       $results .= "NO DMARC record for domain $senderdomain, mail from email address &lt;$senderemailaddress&gt;" . $CRLF;
    }

    return $results;
}  


?>